package net.wanji.opt.servicev2.report.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.entity.GreenwaveHistPO;
import net.wanji.databus.dao.mapper.GreenwaveHistMapper;
import net.wanji.opt.dao.mapper.report.GreenWaveWeekDataMapper;
import net.wanji.opt.entity.report.ConfigPeakHours;
import net.wanji.opt.entity.report.GreenWaveWeekData;
import net.wanji.opt.servicev2.report.GreenWaveWeekDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.WeekFields;
import java.util.*;

@Service
@Slf4j
public class GreenWaveWeekDataServiceImpl extends ServiceImpl<GreenWaveWeekDataMapper, GreenWaveWeekData>
        implements GreenWaveWeekDataService {
    @Autowired
    private GreenWaveWeekDataService greenWaveWeekDataService;
    @Autowired
    private GreenWaveWeekDataMapper greenWaveWeekDataMapper;
    @Autowired
    private GreenwaveHistMapper greenwaveHistMapper;

    private static final int TIME_GAP_THRESHOLD = 900; // 时间间隔阈值，单位为秒
    private static final int MINUTES_PER_RECORD = 5; // 每条记录对应的时间，单位为分钟
    private static final int SECONDS_PER_MINUTE = 60; // 每分钟的秒数

    @Override
    public void insertGreenWaveWeekData() {
        List<ConfigPeakHours> configPeakHours = greenWaveWeekDataMapper.selectConfigPeakHoursAll();
        // 获取上周的开始时间（周一 00:00:00）
        LocalDateTime startOfLastWeek = getStartOfLastWeek();
        // 获取上周的结束时间（周日 23:59:59）
        LocalDateTime endOfLastWeek = getEndOfLastWeek();

        // 格式化输出
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String startDate = startOfLastWeek.format(formatter);
        String endDate = endOfLastWeek.format(formatter);
        for (ConfigPeakHours configPeakHour : configPeakHours) {
            String startTime = configPeakHour.getStartTime();
            String endTime = configPeakHour.getEndTime();
            List<GreenWaveWeekData> greenWaveWeekDataList = greenWaveWeekDataMapper.selectGreenWaveWeekDataByTime(startDate, endDate, startTime, endTime);
            greenWaveWeekDataList.forEach(x -> {
                x.setPeakType(String.valueOf(configPeakHour.getPeakType()));
                x.setDayType(String.valueOf(configPeakHour.getDayType()));
                x.setStartTime(startDate);
                x.setEndTime(endDate);
                Integer dir = getDir(x.getRoadDirection());
                Integer flow = greenWaveWeekDataMapper.getFlowByTime(x.getGreenId(), dir, startDate, endDate, startTime, endTime);
                x.setFlow(flow);
                x.setYearWeek(getWeekNumberOfYear(startOfLastWeek));
                x.setInsertTime(new Date());
                GreenWaveWeekData greenWaveWeekData = getGreenWaveWeekData(x, startTime, endTime);
            });
            greenWaveWeekDataService.insertOrUpdateBatch(greenWaveWeekDataList);
            log.info("周总体拥堵高峰期概况批量保存或更新成功！");
        }
        List<GreenWaveWeekData> greenWaveWeekDataList = greenWaveWeekDataMapper.selectGreenWaveWeekData(startDate, endDate);
        greenWaveWeekDataList.forEach(x -> {
            x.setPeakType("3");
            x.setDayType("1");
            x.setStartTime(startDate);
            x.setEndTime(endDate);
            Integer dir = getDir(x.getRoadDirection());
            Integer flow = greenWaveWeekDataMapper.getFlow(x.getGreenId(), dir, startDate, endDate);
            x.setFlow(flow);
            x.setYearWeek(getWeekNumberOfYear(startOfLastWeek));
            x.setInsertTime(new Date());
            GreenWaveWeekData greenWaveWeekData = getGreenWaveWeekData(x, null, null);

        });
        greenWaveWeekDataService.insertOrUpdateBatch(greenWaveWeekDataList);
        log.info("周总体拥堵全天概况批量保存或更新成功！");
    }

    private GreenWaveWeekData getGreenWaveWeekData(GreenWaveWeekData x, String startTime, String endTime) {
        List<GreenwaveHistPO> greenwaveHistPOS = greenWaveWeekDataMapper.selectAllByTime(x.getGreenId(), x.getRoadDirection(), x.getStartTime(), x.getEndTime(), startTime, endTime);
        if (ObjectUtils.isEmpty(greenwaveHistPOS)) {
            x.setCongestIndex(0.0);
            x.setMaxCongestIndex(0.0);
            x.setCongestCount(0);
            x.setMaxCongestDuration(0.0);
            return x;
        }
        //拥堵次数
        int count = 1;
        //每次拥堵条数
        int mCount = 1;
        //最大拥堵指数
        double maxCongestIndex = 0;
        //每次拥堵指数和
        double congestIndex = greenwaveHistPOS.get(0).getTrafficIndex();
        //拥堵指数和
        double congestIndexAll = 0;
        //最大拥堵持续时长
        int maxCongestDuration = 0;
        int secondCongestDuration = 0;
        //拥堵时间
        Date congestStartTime = greenwaveHistPOS.get(0).getStartTime();
        Date congestEndTime = null;
        Date maxCongestStartTime = null;
        Date maxCongestEndTime = null;
        Date secondCongestStartTime = null;
        Date secondCongestEndTime = null;
        Calendar calendar = Calendar.getInstance();
        //判断是否只有一条拥堵数据
        if (greenwaveHistPOS.size() == 1) {
            maxCongestIndex = greenwaveHistPOS.get(0).getTrafficIndex();
            congestIndexAll = maxCongestIndex;
            maxCongestDuration = MINUTES_PER_RECORD * SECONDS_PER_MINUTE;
            // 使用 Calendar 添加 5 分钟

            calendar.setTime(congestStartTime);
            calendar.add(Calendar.MINUTE, 5);
            // 获取更新后的时间
            maxCongestEndTime = calendar.getTime();

            maxCongestStartTime = congestStartTime;
        } else {
            for (int i = 1; i < greenwaveHistPOS.size(); i++) {
                GreenwaveHistPO greenwaveHistPONext = greenwaveHistPOS.get(i);
                GreenwaveHistPO greenwaveHistPO = greenwaveHistPOS.get(i - 1);

                if (greenwaveHistPONext.getBatchTime() - greenwaveHistPO.getBatchTime() > TIME_GAP_THRESHOLD) {

                    //获取最大拥堵指数
                    if (congestIndex / mCount > maxCongestIndex) {
                        maxCongestIndex = congestIndex / mCount;
                    }
                    //拥堵指数和
                    congestIndexAll += congestIndex / mCount;
                    //拥堵持续时间
                    int congestDuration = mCount * MINUTES_PER_RECORD * SECONDS_PER_MINUTE;
                    calendar.setTime(greenwaveHistPO.getStartTime());
                    calendar.add(Calendar.MINUTE, MINUTES_PER_RECORD);
                    // 获取更新后的时间
                    congestEndTime = calendar.getTime();
                    //最大拥堵持续时间
                    //最大拥堵持续时间
                    if (maxCongestDuration < congestDuration) {
                        maxCongestDuration = congestDuration;
                        maxCongestStartTime = congestStartTime;
                        maxCongestEndTime = congestEndTime;
                    }else if (secondCongestDuration < congestDuration) {
                        secondCongestDuration = congestDuration;
                        secondCongestStartTime = congestStartTime;
                        secondCongestEndTime = congestEndTime;
                    }


                    //新一次拥堵
                    count++;
                    mCount = 1;
                    congestIndex = greenwaveHistPONext.getTrafficIndex();
                    congestStartTime = greenwaveHistPONext.getStartTime();
                    if (i == greenwaveHistPOS.size() - 1) {
                        //获取最大拥堵指数
                        if (congestIndex / mCount > maxCongestIndex) {
                            maxCongestIndex = congestIndex / mCount;
                        }
                        //拥堵指数和
                        congestIndexAll += congestIndex / mCount;
                        //拥堵持续时间
                        congestDuration = mCount * MINUTES_PER_RECORD * SECONDS_PER_MINUTE;
                        // 使用 Calendar 添加 5 分钟
                        calendar.setTime(congestStartTime);
                        calendar.add(Calendar.MINUTE, MINUTES_PER_RECORD);
                        // 获取更新后的时间
                        congestEndTime = calendar.getTime();
                        //最大拥堵持续时间
                        if (maxCongestDuration < congestDuration) {
                            maxCongestDuration = congestDuration;
                            maxCongestStartTime = congestStartTime;
                            maxCongestEndTime = congestEndTime;
                        }else if (secondCongestDuration < congestDuration) {
                            secondCongestDuration = congestDuration;
                            secondCongestStartTime = congestStartTime;
                            secondCongestEndTime = congestEndTime;
                        }

                    }
                } else {
                    congestIndex += greenwaveHistPONext.getTrafficIndex();
                    mCount++;
                    if (i == greenwaveHistPOS.size() - 1) {
                        //获取最大拥堵指数
                        if (congestIndex / mCount > maxCongestIndex) {
                            maxCongestIndex = congestIndex / mCount;
                        }
                        //拥堵指数和
                        congestIndexAll += congestIndex / mCount;
                        //拥堵持续时间
                        int congestDuration = mCount * MINUTES_PER_RECORD * SECONDS_PER_MINUTE;
                        // 获取更新后的时间
                        calendar.setTime(greenwaveHistPONext.getStartTime());
                        calendar.add(Calendar.MINUTE, MINUTES_PER_RECORD);
                        // 获取更新后的时间
                        congestEndTime = calendar.getTime();
                        //最大拥堵持续时间
                        if (maxCongestDuration < congestDuration) {
                            maxCongestDuration = congestDuration;
                            maxCongestStartTime = congestStartTime;
                            maxCongestEndTime = congestEndTime;
                        }else if (secondCongestDuration < congestDuration) {
                            secondCongestDuration = congestDuration;
                            secondCongestStartTime = congestStartTime;
                            secondCongestEndTime = congestEndTime;
                        }

                    }
                }
            }
        }
        x.setCongestIndex(congestIndexAll / count);
        x.setMaxCongestIndex(maxCongestIndex);
        x.setCongestCount(count);
        x.setMaxCongestDuration((double) maxCongestDuration);
        String formattedTimeRange = formatTimeRange(maxCongestStartTime, maxCongestEndTime);
        x.setMaxCongestionPeriod(formattedTimeRange);
        if (!ObjectUtils.isEmpty(secondCongestStartTime) && !ObjectUtils.isEmpty(secondCongestEndTime)) {
            String formattedTimeRange1 = formatTimeRange(secondCongestStartTime, secondCongestEndTime);
            if (!formattedTimeRange.equals(formattedTimeRange1)) {
                x.setSecondCongestionPeriod(formattedTimeRange1);
            }
        }
        return x;

    }

    @Override
    public void insertOrUpdateBatch(List<GreenWaveWeekData> greenWaveWeekDataList) {
        greenWaveWeekDataMapper.insertOrUpdateBatch(greenWaveWeekDataList);
    }


    private Integer getDir(String roadDirection) {
        switch (roadDirection) {
            case "e2w":
                return 3;
            case "w2e":
                return 7;
            case "s2n":
                return 5;
            case "n2s":
                return 1;
            default:
                return 3;
        }
    }

    public static Integer getWeekNumberOfYear(LocalDateTime date) {
        // 使用默认区域设置获取周字段
        WeekFields weekFields = WeekFields.of(Locale.getDefault());

        // 获取年份
        int year = date.get(weekFields.weekBasedYear());

        // 获取周数
        int weekNumber = date.get(weekFields.weekOfWeekBasedYear());

        // 拼接成 "yyyyw" 格式
        String format = String.format("%d%d", year, weekNumber);
        return Integer.valueOf(format);
    }

    /*
     * 获取日期yyyy-MM-dd
     * */
    public List<String> getLastWeeKDays() {
        LocalDate today = LocalDate.now();
        LocalDate lastMonday = today.with(DayOfWeek.MONDAY).minusWeeks(1);

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        List<String> dayList = new ArrayList<>();
        for (int i = 0; i < 7; i++) {
            LocalDate day = lastMonday.plusDays(i);
            dayList.add(day.format(formatter));
        }
        return dayList;
    }

    /**
     * 获取上周的开始时间（周一 00:00:00）
     */
    public static LocalDateTime getStartOfLastWeek() {
        // 获取当前日期
        LocalDateTime now = LocalDateTime.now();
        // 找到上周一
        LocalDateTime lastMonday = now.with(DayOfWeek.MONDAY).minusWeeks(1)
                .with(LocalTime.MIN); // 设置时间为 00:00:00
        return lastMonday;
    }

    /**
     * 获取上周的结束时间（周日 23:59:59）
     */
    public static LocalDateTime getEndOfLastWeek() {
        // 获取当前日期
        LocalDateTime now = LocalDateTime.now();
        // 找到上周日
        LocalDateTime lastSunday = now.with(DayOfWeek.SUNDAY).minusWeeks(1)
                .with(LocalTime.MAX); // 设置时间为 23:59:59
        return lastSunday;
    }

    public static String formatTimeRange(Date startTime, Date endTime) {
        // 将 Date 转换为 LocalDateTime
        LocalDateTime startDateTime = startTime.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
        LocalDateTime endDateTime = endTime.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();

        // 获取星期几
        DayOfWeek dayOfWeek = startDateTime.getDayOfWeek();
        String weekDay = getWeekDayName(dayOfWeek);

        // 格式化时间为 HH:mm
        DateTimeFormatter timeFormatter = DateTimeFormatter.ofPattern("HH:mm");
        String startTimeStr = startDateTime.format(timeFormatter);
        String endTimeStr = endDateTime.format(timeFormatter);

        // 拼接结果
        return weekDay + " " + startTimeStr + "~" + endTimeStr;
    }

    private static String getWeekDayName(DayOfWeek dayOfWeek) {
        // 映射 DayOfWeek 到中文星期名称
        switch (dayOfWeek) {
            case MONDAY:
                return "周一";
            case TUESDAY:
                return "周二";
            case WEDNESDAY:
                return "周三";
            case THURSDAY:
                return "周四";
            case FRIDAY:
                return "周五";
            case SATURDAY:
                return "周六";
            case SUNDAY:
                return "周日";
            default:
                return "";
        }
    }
}