package net.wanji.opt.servicev2.strategy.impl;

import com.alibaba.nacos.common.utils.CollectionUtils;
import com.alibaba.nacos.common.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.opt.dao.mapper.strategy.StrategyPriorityMapper;
import net.wanji.opt.entity.strategy.StrategyPriorityConfig;
import net.wanji.opt.entity.strategy.StrategyPriorityDailyInfo;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import net.wanji.opt.entity.strategy.dto.StrategyPriorityGroup;
import net.wanji.opt.servicev2.strategy.StrategyPriorityService;
import net.wanji.opt.synthesis.pojo.StrategyFactoryEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
* <p>
* 策略优先级日计划详情表 服务实现类
* </p>
*
* @author fengyi
* @since 2025-03-28
*/
@Service
@Slf4j
 public class StrategyPriorityServiceImpl extends ServiceImpl<StrategyPriorityMapper, StrategyPriorityDailyInfo> implements StrategyPriorityService {

	 @Resource
	 private StrategyPriorityMapper strategyPriorityMapper;

	
	 /**
	 * 策略优先级日计划详情表详情
	 * @param id
	 * @return
	 */
	 @Override
	 public StrategyPriorityDailyInfo info(Long id) {
	
	 	return getById(id);
	 }
	
	 /**
	 * 策略优先级日计划详情表新增
	 * @param param 根据需要进行传值
	 * @return
	 */
	 @Override
	 public void add(StrategyPriorityDailyInfo param) {
	
	 	save(param);
	 }
	
	 /**
	 * 策略优先级日计划详情表修改
	 * @param param 根据需要进行传值
	 * @return
	 */
	 @Override
	 public void modify(StrategyPriorityDailyInfo param) {
	
	 	updateById(param);
	 }
	
	 /**
	 * 策略优先级日计划详情表删除(单个条目)
	 * @param id
	 * @return
	 */
	 @Override
	 public void remove(Long id) {
	 	removeById(id);
	 }
	
	 /**
	 * 策略优先级日计划详情表删除(多个条目)
	 * @param ids
	 * @return
	 */
	 @Override
	 public void removes(List<Long> ids) {
	
	 	removeByIds(ids);
	 }


	/**
	 * 策略路口列表
	 * @param
	 * @return
	 */
	@Override
	public  List<StrategyPriorityDailyInfo> pageStrategyCrossList() {
		 List<StrategyPriorityDailyInfo> allList= strategyPriorityMapper.pageStrategyCrossList();

		 return allList ;
	 }

	/**
	 * 策略名称
	 * @param
	 * @return
	 */
	@Override
	public List<StrategyFactoryEntity> getStrategyList() {
		List<StrategyFactoryEntity> strategyList= strategyPriorityMapper.getStrategyList();
		return strategyList;
	}
	/**
	 * 场景
	 * @param
	 * @return
	 */
	@Override
	public List<StrategyFactoryEntity> getSceneList() {
		List<StrategyFactoryEntity> strategyList= strategyPriorityMapper.getSceneList();
		return strategyList;
	}
	/**
	 * 厂商
	 * @param
	 * @return
	 */
	@Override
	public List<StrategyFactoryEntity> getCompanyList() {
		List<StrategyFactoryEntity> strategyList=strategyPriorityMapper.getCompanyList();
		return strategyList;
	}

	@Override
	@Transactional
	public void savePriority(List<StrategyPriorityGroup> dataList) {
		try {
			List<StrategyPriorityConfig> saveList = new ArrayList<>();
			for (StrategyPriorityGroup group : dataList) {
				String label = group.getLabel();
				Integer labelCode = group.getLabelCode();
				//路口id
				String crossId = group.getCrossId();
				//先判断表里有没有数据，有的话删除t_strategy_priority_config
				List<StrategyPriorityConfig> priorityConfig = strategyPriorityMapper.selectPriorityTable(crossId);
				if (priorityConfig.size() > 0) {
					strategyPriorityMapper.deletePriorityConfig(crossId);
				}

				if (CollectionUtils.isNotEmpty(group.getData())) {
					for (StrategyPriorityConfig item : group.getData()) {
						StrategyPriorityConfig strategyPriorityConfig = new StrategyPriorityConfig();
						strategyPriorityConfig.setCrossId(crossId);
						//分组id
						strategyPriorityConfig.setGroupId(labelCode);
						//分组名称
						strategyPriorityConfig.setGroupName(label);
						//策略编号
						strategyPriorityConfig.setStrategyNo(item.getStrategyNo());
						//优先级
						strategyPriorityConfig.setPriority(item.getPriority());

						saveList.add(strategyPriorityConfig);
					}
				}
			}

			strategyPriorityMapper.savePriorityInsert(saveList);
		} catch (Exception e) {
			log.error("{} savePriority", this.getClass().getSimpleName(), e);
			throw  e;
		}
	}

	@Override
	@Transactional
	public List<StrategyFactoryEntity> getPriorityData(String crossId) {

		try {
			//先获取策略编号
			List<String> strategyNo = strategyPriorityMapper.getstrategyNo(crossId);
			List<StrategyFactoryEntity> priorityList = strategyPriorityMapper.getPriorityConfigData(strategyNo,crossId);
			return  priorityList;
		}catch (Exception e){
			log.error("{} getPriorityData", this.getClass().getSimpleName(), e);
			throw  e;
		}

	}
}
